@@ -49,14 +49,17 @@ class EC2Client(BaseClient):
         security_groups=None, key_name=None, instance_type=None,
         user_data=None, availability_zone=None, kernel_id=None,
         ramdisk_id=None):
-        """Run new instances."""
+        """Run new instances.
+
+        TODO: blockDeviceMapping, monitoring, subnetId
+        """
         params = {"ImageId": image_id, "MinCount": str(min_count),
                   "MaxCount": str(max_count)}
+        if key_name is not None:
+            params["KeyName"] = key_name
         if security_groups is not None:
             for i, name in enumerate(security_groups):
                 params["SecurityGroup.%d" % (i + 1)] = name
-        if key_name is not None:
-            params["KeyName"] = key_name
         if user_data is not None:
             params["UserData"] = b64encode(user_data)
         if instance_type is not None:
@@ -340,7 +343,10 @@ class EC2Client(BaseClient):
         return d.addCallback(self.parser.truth_return)
 
     def describe_snapshots(self, *snapshot_ids):
-        """Describe available snapshots."""
+        """Describe available snapshots.
+
+        TODO: ownerSet, restorableBySet
+        """
         snapshot_set = {}
         for pos, snapshot_id in enumerate(snapshot_ids):
             snapshot_set["SnapshotId.%d" % (pos + 1)] = snapshot_id
@@ -351,7 +357,10 @@ class EC2Client(BaseClient):
         return d.addCallback(self.parser.snapshots)
 
     def create_snapshot(self, volume_id):
-        """Create a new snapshot of an existing volume."""
+        """Create a new snapshot of an existing volume.
+
+        TODO: description
+        """
         query = self.query_factory(
             action="CreateSnapshot", creds=self.creds, endpoint=self.endpoint,
             other_params={"VolumeId": volume_id})
@@ -379,7 +388,7 @@ class EC2Client(BaseClient):
         """Returns information about key pairs available."""
         keypairs = {}
         for index, keypair_name in enumerate(keypair_names):
-            keypairs["KeyPair.%d" % (index + 1)] = keypair_name
+            keypairs["KeyName.%d" % (index + 1)] = keypair_name
         query = self.query_factory(
             action="DescribeKeyPairs", creds=self.creds,
             endpoint=self.endpoint, other_params=keypairs)
@@ -418,6 +427,9 @@ class EC2Client(BaseClient):
 
         @return: A L{Deferred} firing with a L{model.Keypair} instance if
             successful.
+
+        TODO: there is no corresponding method in the 2009-11-30 version
+             of the ec2 wsdl. Delete this?
         """
         query = self.query_factory(
             action="ImportKeyPair", creds=self.creds, endpoint=self.endpoint,
@@ -530,31 +542,37 @@ class Parser(object):
         @param instance_data: An XML node containing instance data.
         @param reservation: The L{Reservation} associated with the instance.
         @return: An L{Instance}.
+
+        TODO: reason, platform, monitoring, subnetId, vpcId, privateIpAddress,
+              ipAddress, stateReason, architecture, rootDeviceName,
+              blockDeviceMapping, instanceLifecycle, spotInstanceRequestId.
         """
         instance_id = instance_data.findtext("instanceId")
         instance_state = instance_data.find(
             "instanceState").findtext("name")
-        instance_type = instance_data.findtext("instanceType")
-        image_id = instance_data.findtext("imageId")
         private_dns_name = instance_data.findtext("privateDnsName")
         dns_name = instance_data.findtext("dnsName")
+        private_ip_address = instance_data.findtext("privateIpAddress")
+        ip_address = instance_data.findtext("ipAddress")
         key_name = instance_data.findtext("keyName")
         ami_launch_index = instance_data.findtext("amiLaunchIndex")
-        launch_time = instance_data.findtext("launchTime")
-        placement = instance_data.find("placement").findtext(
-            "availabilityZone")
         products = []
         product_codes = instance_data.find("productCodes")
         if product_codes is not None:
             for product_data in instance_data.find("productCodes"):
                 products.append(product_data.text)
+        instance_type = instance_data.findtext("instanceType")
+        launch_time = instance_data.findtext("launchTime")
+        placement = instance_data.find("placement").findtext(
+            "availabilityZone")
         kernel_id = instance_data.findtext("kernelId")
         ramdisk_id = instance_data.findtext("ramdiskId")
+        image_id = instance_data.findtext("imageId")
         instance = model.Instance(
             instance_id, instance_state, instance_type, image_id,
-            private_dns_name, dns_name, key_name, ami_launch_index,
-            launch_time, placement, products, kernel_id, ramdisk_id,
-            reservation=reservation)
+            private_dns_name, dns_name, private_ip_address, ip_address,
+            key_name, ami_launch_index, launch_time, placement, products,
+            kernel_id, ramdisk_id, reservation=reservation)
         return instance
 
     def describe_instances(self, xml_bytes):
@@ -602,7 +620,8 @@ class Parser(object):
         Parse the reservations XML payload that is returned from an AWS
         RunInstances API call.
 
-        @param xml_bytes: raw XML payload from AWS.
+        @param xml_bytes: raw XML bytes with a C{RunInstancesResponse} root
+            element.
         """
         root = XML(xml_bytes)
         # Get the security group information.
@@ -625,18 +644,20 @@ class Parser(object):
         @param xml_bytes: XML bytes with a C{TerminateInstancesResponse} root
             element.
         @return: An iterable of C{tuple} of (instanceId, previousState,
-            shutdownState) for the ec2 instances that where terminated.
+            currentState) for the ec2 instances that where terminated.
         """
         root = XML(xml_bytes)
         result = []
         # May be a more elegant way to do this:
-        for instance in root.find("instancesSet"):
-            instanceId = instance.findtext("instanceId")
-            previousState = instance.find("previousState").findtext(
-                "name")
-            shutdownState = instance.find("shutdownState").findtext(
-                "name")
-            result.append((instanceId, previousState, shutdownState))
+        instances = root.find("instancesSet")
+        if instances is not None:
+            for instance in instances:
+                instanceId = instance.findtext("instanceId")
+                previousState = instance.find("previousState").findtext(
+                    "name")
+                currentState = instance.find("currentState").findtext(
+                    "name")
+                result.append((instanceId, previousState, currentState))
         return result
 
     def describe_security_groups(self, xml_bytes):
@@ -658,9 +679,20 @@ class Parser(object):
             if ip_permissions is None:
                 ip_permissions = ()
             for ip_permission in ip_permissions:
+
+                # openstack doesn't handle self authorized groups properly
+                # XXX this is an upstream problem and should be addressed there
+                # lp bug #829609
                 ip_protocol = ip_permission.findtext("ipProtocol")
-                from_port = int(ip_permission.findtext("fromPort"))
-                to_port = int(ip_permission.findtext("toPort"))
+                from_port = ip_permission.findtext("fromPort")
+                to_port = ip_permission.findtext("toPort")
+
+                if from_port:
+                    from_port = int(from_port)
+
+                if to_port:
+                    to_port = int(to_port)
+
                 for groups in ip_permission.findall("groups/item") or ():
                     user_id = groups.findtext("userId")
                     group_name = groups.findtext("groupName")
@@ -697,15 +729,17 @@ class Parser(object):
         @param xml_bytes: XML bytes with a C{DescribeVolumesResponse} root
             element.
         @return: A list of L{Volume} instances.
+
+        TODO: attachementSetItemResponseType#deleteOnTermination
         """
         root = XML(xml_bytes)
         result = []
         for volume_data in root.find("volumeSet"):
             volume_id = volume_data.findtext("volumeId")
             size = int(volume_data.findtext("size"))
-            status = volume_data.findtext("status")
-            availability_zone = volume_data.findtext("availabilityZone")
             snapshot_id = volume_data.findtext("snapshotId")
+            availability_zone = volume_data.findtext("availabilityZone")
+            status = volume_data.findtext("status")
             create_time = volume_data.findtext("createTime")
             create_time = datetime.strptime(
                 create_time[:19], "%Y-%m-%dT%H:%M:%S")
@@ -715,8 +749,8 @@ class Parser(object):
             result.append(volume)
             for attachment_data in volume_data.find("attachmentSet"):
                 instance_id = attachment_data.findtext("instanceId")
-                status = attachment_data.findtext("status")
                 device = attachment_data.findtext("device")
+                status = attachment_data.findtext("status")
                 attach_time = attachment_data.findtext("attachTime")
                 attach_time = datetime.strptime(
                     attach_time[:19], "%Y-%m-%dT%H:%M:%S")
@@ -735,10 +769,10 @@ class Parser(object):
         root = XML(xml_bytes)
         volume_id = root.findtext("volumeId")
         size = int(root.findtext("size"))
+        snapshot_id = root.findtext("snapshotId")
+        availability_zone = root.findtext("availabilityZone")
         status = root.findtext("status")
         create_time = root.findtext("createTime")
-        availability_zone = root.findtext("availabilityZone")
-        snapshot_id = root.findtext("snapshotId")
         create_time = datetime.strptime(
             create_time[:19], "%Y-%m-%dT%H:%M:%S")
         volume = model.Volume(
@@ -752,6 +786,9 @@ class Parser(object):
         @param xml_bytes: XML bytes with a C{DescribeSnapshotsResponse} root
             element.
         @return: A list of L{Snapshot} instances.
+
+        TODO: ownersSet, restorableBySet, ownerId, volumeSize, description,
+              ownerAlias.
         """
         root = XML(xml_bytes)
         result = []
@@ -775,6 +812,8 @@ class Parser(object):
         @param xml_bytes: XML bytes with a C{CreateSnapshotResponse} root
             element.
         @return: The L{Snapshot} instance created.
+
+        TODO: ownerId, volumeSize, description.
         """
         root = XML(xml_bytes)
         snapshot_id = root.findtext("snapshotId")
@@ -794,6 +833,8 @@ class Parser(object):
         @param xml_bytes: XML bytes with a C{AttachVolumeResponse} root
             element.
         @return: a C{dict} with status and attach_time keys.
+
+        TODO: volumeId, instanceId, device
         """
         root = XML(xml_bytes)
         status = root.findtext("status")
@@ -834,7 +875,11 @@ class Parser(object):
         return model.Keypair(key_name, key_fingerprint, key_material)
 
     def import_keypair(self, xml_bytes, key_material):
-        """Extract the key name and the fingerprint from the result."""
+        """Extract the key name and the fingerprint from the result.
+
+        TODO: there is no corresponding method in the 2009-11-30 version
+             of the ec2 wsdl. Delete this?
+        """
         keypair_data = XML(xml_bytes)
         key_name = keypair_data.findtext("keyName")
         key_fingerprint = keypair_data.findtext("keyFingerprint")
@@ -870,6 +915,8 @@ class Parser(object):
         @param xml_bytes: XML bytes with a C{DescribeAvailibilityZonesResponse}
             root element.
         @return: a C{list} of L{AvailabilityZone}.
+
+        TODO: regionName, messageSet
         """
         results = []
         root = XML(xml_bytes)
@@ -889,7 +936,7 @@ class Query(BaseQuery):
                  *args, **kwargs):
         """Create a Query to submit to EC2."""
         super(Query, self).__init__(*args, **kwargs)
-        # Currently, txAWS only supports version 2008-12-01
+        # Currently, txAWS only supports version 2009-11-30
         if api_version is None:
             api_version = version.ec2_api
         self.params = {
@@ -952,7 +999,9 @@ class Signature(object):
 
     @ivar creds: The L{AWSCredentials} to use to compute the signature.
     @ivar endpoint: The {AWSServiceEndpoint} to consider.
-    @ivar params: A C{dict} of parameters to consider.
+    @ivar params: A C{dict} of parameters to consider. They should be byte
+        strings, but unicode strings are supported and will be encoded in
+        UTF-8.
     """
 
     def __init__(self, creds, endpoint, params):
@@ -1002,9 +1051,11 @@ class Signature(object):
     def encode(self, string):
         """Encode a_string as per the canonicalisation encoding rules.
 
-        See the AWS dev reference page 90 (2008-12-01 version).
+        See the AWS dev reference page 186 (2009-11-30 version).
         @return: a_string encoded.
         """
+        if isinstance(string, unicode):
+            string = string.encode("utf-8")
         return quote(string, safe="~")
 
     def sorted_params(self):
