@@ -40,14 +40,17 @@ class InstanceTestCase(TXAWSTestCase):
 
     def test_instance_creation(self):
         instance = model.Instance(
-            "id1", "running", "type", "id2", "dns1", "dns2", "key", "ami",
-            "time", "placement", ["prod1", "prod2"], "id3", "id4")
+            "id1", "running", "type", "id2", "dns1", "dns2", "ip1",
+            "ip2", "key", "ami", "time", "placement",
+            ["prod1", "prod2"], "id3", "id4")
         self.assertEquals(instance.instance_id, "id1")
         self.assertEquals(instance.instance_state, "running")
         self.assertEquals(instance.instance_type, "type")
         self.assertEquals(instance.image_id, "id2")
         self.assertEquals(instance.private_dns_name, "dns1")
         self.assertEquals(instance.dns_name, "dns2")
+        self.assertEquals(instance.private_ip_address, "ip1")
+        self.assertEquals(instance.ip_address, "ip2")
         self.assertEquals(instance.key_name, "key")
         self.assertEquals(instance.ami_launch_index, "ami")
         self.assertEquals(instance.launch_time, "time")
@@ -139,7 +142,8 @@ class EC2ClientTestCase(TXAWSTestCase):
 
             def submit(self):
                 return succeed(
-                    payload.sample_describe_availability_zones_multiple_results)
+                    payload.
+                        sample_describe_availability_zones_multiple_results)
 
         def check_parsed_availability_zones(results):
             self.assertEquals(len(results), 3)
@@ -179,6 +183,8 @@ class EC2ClientInstancesTestCase(TXAWSTestCase):
         self.assertEquals(
             instance.dns_name,
             "ec2-75-101-245-65.compute-1.amazonaws.com")
+        self.assertEquals(instance.private_ip_address, "10.0.0.1")
+        self.assertEquals(instance.ip_address, "75.101.245.65")
         self.assertEquals(instance.key_name, "keyname")
         self.assertEquals(instance.ami_launch_index, "0")
         self.assertEquals(instance.launch_time, "2009-04-27T02:23:18.000Z")
@@ -207,6 +213,8 @@ class EC2ClientInstancesTestCase(TXAWSTestCase):
         self.assertEquals(
             instance.dns_name,
             "ec2-75-101-245-65.compute-1.amazonaws.com")
+        self.assertEquals(instance.private_ip_address, "10.0.0.1")
+        self.assertEquals(instance.ip_address, "75.101.245.65")
         self.assertEquals(instance.key_name, None)
         self.assertEquals(instance.ami_launch_index, None)
         self.assertEquals(instance.launch_time, "2009-04-27T02:23:18.000Z")
@@ -522,6 +530,38 @@ class EC2ClientSecurityGroupsTestCase(TXAWSTestCase):
         d = ec2.describe_security_groups("WebServers")
         return d.addCallback(check_result)
 
+    def test_describe_security_groups_with_openstack(self):
+        """
+        L{EC2Client.describe_security_groups} can work with openstack
+        responses, which may lack proper port information for
+        self-referencing group. Verifying that the response doesn't
+        cause an internal error, workaround for nova launchpad bug
+        #829609.
+        """
+        class StubQuery(object):
+
+            def __init__(stub, action="", creds=None, endpoint=None,
+                         other_params={}):
+                self.assertEqual(action, "DescribeSecurityGroups")
+                self.assertEqual(creds.access_key, "foo")
+                self.assertEqual(creds.secret_key, "bar")
+                self.assertEqual(other_params, {"GroupName.1": "WebServers"})
+
+            def submit(self):
+                return succeed(
+                    payload.sample_describe_security_groups_with_openstack)
+
+        def check_result(security_groups):
+            [security_group] = security_groups
+            self.assertEquals(security_group.name, "WebServers")
+            self.assertEqual(
+                security_group.allowed_groups[0].group_name, "WebServers")
+
+        creds = AWSCredentials("foo", "bar")
+        ec2 = client.EC2Client(creds, query_factory=StubQuery)
+        d = ec2.describe_security_groups("WebServers")
+        return d.addCallback(check_result)
+
     def test_create_security_group(self):
         """
         L{EC2Client.create_security_group} returns a C{Deferred} that
@@ -1256,7 +1296,7 @@ class EC2ClientEBSTestCase(TXAWSTestCase):
                 self.assertEqual("foo", creds)
                 self.assertEquals(
                     other_params,
-                    {"KeyPair.1": "gsg-keypair"})
+                    {"KeyName.1": "gsg-keypair"})
 
             def submit(self):
                 return succeed(payload.sample_single_describe_keypairs_result)
@@ -1521,7 +1561,7 @@ class QueryTestCase(TXAWSTestCase):
             {"AWSAccessKeyId": "foo",
              "Action": "DescribeInstances",
              "SignatureVersion": "2",
-             "Version": "2008-12-01"})
+             "Version": "2009-11-30"})
 
     def test_init_other_args_are_params(self):
         query = client.Query(
@@ -1535,7 +1575,7 @@ class QueryTestCase(TXAWSTestCase):
              "InstanceId.0": "12345",
              "SignatureVersion": "2",
              "Timestamp": "2007-11-12T13:14:15Z",
-             "Version": "2008-12-01"})
+             "Version": "2009-11-30"})
 
     def test_no_timestamp_if_expires_in_other_params(self):
         """
@@ -1553,7 +1593,7 @@ class QueryTestCase(TXAWSTestCase):
              "Action": "DescribeInstances",
              "SignatureVersion": "2",
              "Expires": "2007-11-12T13:14:15Z",
-             "Version": "2008-12-01"})
+             "Version": "2009-11-30"})
 
     def test_sign(self):
         query = client.Query(
@@ -1561,7 +1601,7 @@ class QueryTestCase(TXAWSTestCase):
             endpoint=self.endpoint,
             time_tuple=(2007, 11, 12, 13, 14, 15, 0, 0, 0))
         query.sign()
-        self.assertEqual("aDmLr0Ktjsmt17UJD/EZf6DrfKWT1JW0fq2FDUCOPic=",
+        self.assertEqual("G4c2NtQaFNhWWT8EWPVIIOpHVr0mGUYwJVYss9krsMU=",
             query.params["Signature"])
 
     def test_old_sign(self):
@@ -1572,7 +1612,7 @@ class QueryTestCase(TXAWSTestCase):
             other_params={"SignatureVersion": "1"})
         query.sign()
         self.assertEqual(
-            "MBKyHoxqCd/lBQLVkCZYpwAtNJg=", query.params["Signature"])
+            "9xP+PIs/3QXW+4mWX6WGR4nGqfE=", query.params["Signature"])
 
     def test_unsupported_sign(self):
         query = client.Query(
@@ -1703,6 +1743,15 @@ class SignatureTestCase(TXAWSTestCase):
         signature = client.Signature(self.creds, self.endpoint, self.params)
         self.assertEqual("a%20space", signature.encode("a space"))
 
+    def test_encode_unicode(self):
+        """
+        L{Signature.encode} accepts unicode strings and encode them un UTF-8.
+        """
+        signature = client.Signature(self.creds, self.endpoint, self.params)
+        self.assertEqual(
+            "f%C3%A9e",
+            signature.encode(u"f\N{LATIN SMALL LETTER E WITH ACUTE}e"))
+
     def test_canonical_query(self):
         signature = client.Signature(self.creds, self.endpoint, self.params)
         time_tuple = (2007, 11, 12, 13, 14, 15, 0, 0, 0)
@@ -1711,15 +1760,16 @@ class SignatureTestCase(TXAWSTestCase):
                             "argwithnovalue": "",
                             "SignatureVersion": "2",
                             "Timestamp": iso8601time(time_tuple),
-                            "Version": "2008-12-01",
+                            "Version": "2009-11-30",
                             "Action": "DescribeInstances",
                             "InstanceId.1": "i-1234"})
         expected_params = ("AWSAccessKeyId=foo&Action=DescribeInstances"
             "&InstanceId.1=i-1234"
             "&SignatureVersion=2&"
-            "Timestamp=2007-11-12T13%3A14%3A15Z&Version=2008-12-01&"
+            "Timestamp=2007-11-12T13%3A14%3A15Z&Version=2009-11-30&"
             "argwithnovalue=&fu%20n=g%2Fames")
-        self.assertEqual(expected_params, signature.get_canonical_query_params())
+        self.assertEqual(
+            expected_params, signature.get_canonical_query_params())
 
     def test_signing_text(self):
         signature = client.Signature(self.creds, self.endpoint, self.params)
@@ -1763,14 +1813,14 @@ class SignatureTestCase(TXAWSTestCase):
         self.params.update({"AWSAccessKeyId": "foo",
                             "fun": "games",
                             "SignatureVersion": "2",
-                            "Version": "2008-12-01",
+                            "Version": "2009-11-30",
                             "Action": "DescribeInstances"})
 
         self.assertEqual([
             ("AWSAccessKeyId", "foo"),
             ("Action", "DescribeInstances"),
             ("SignatureVersion", "2"),
-            ("Version", "2008-12-01"),
+            ("Version", "2009-11-30"),
             ("fun", "games"),
             ], signature.sorted_params())
 
@@ -1962,3 +2012,49 @@ class EC2ClientAddressTestCase(TXAWSTestCase):
         d = ec2.disassociate_address("67.202.55.255")
         d.addCallback(self.assertTrue)
         return d
+
+
+class EC2ParserTestCase(TXAWSTestCase):
+
+    def setUp(self):
+        self.parser = client.Parser()
+
+    def test_ec2_terminate_instances(self):
+        """
+        Given a well formed response from EC2, parse the correct thing.
+        """
+        ec2_xml = """<?xml version="1.0" encoding="UTF-8"?>
+<TerminateInstancesResponse xmlns="http://ec2.amazonaws.com/doc/2008-12-01/">
+    <requestId>d0adc305-7f97-4652-b7c2-6993b2bb8260</requestId>
+    <instancesSet>
+        <item>
+            <instanceId>i-cab0c1aa</instanceId>
+            <currentState>
+                <code>32</code>
+                <name>shutting-down</name>
+            </currentState>
+            <previousState>
+                <code>16</code>
+                <name>running</name>
+            </previousState>
+        </item>
+    </instancesSet>
+</TerminateInstancesResponse>"""
+        ec2_response = self.parser.terminate_instances(ec2_xml)
+        self.assertEquals(
+            [('i-cab0c1aa', 'running', 'shutting-down')], ec2_response)
+
+    def test_nova_terminate_instances(self):
+        """
+        Ensure parser can handle the somewhat non-standard response from nova
+        Note that the bug has been reported in nova here:
+          https://launchpad.net/bugs/862680
+        """
+
+        nova_xml = (
+            '<?xml version="1.0" ?><TerminateInstancesResponse '
+            'xmlns="http://ec2.amazonaws.com/doc/2008-12-01/"><requestId>'
+            '4fe6643d-2346-4add-adb7-a1f61f37c043</requestId>'
+            '<return>true</return></TerminateInstancesResponse>')
+        nova_response = self.parser.terminate_instances(nova_xml)
+        self.assertEquals([], nova_response)
