@@ -122,8 +122,13 @@ class AWSError(Error):
             return self.errors[0]["Message"]
 
 
-
 class AWSResponseParseError(Exception):
     """
     txAWS was unable to parse the server response.
     """
+
+
+class CertsNotFoundError(Exception):
+    """
+    txAWS was not able to find any SSL certificates.
+    """
