@@ -1,6 +1,7 @@
 # Copyright (C) 2008 Tristan Seligmann <mithrandi@mithrandi.net>
 # Copyright (C) 2009 Canonical Ltd
 # Copyright (C) 2009 Duncan McGreggor <oubiwann@adytum.us>
+# Copyright (C) 2012 New Dream Network (DreamHost)
 # Licenced under the txaws licence available at /LICENSE in the txaws source.
 
 """
@@ -15,12 +16,14 @@ import mimetypes
 
 from twisted.web.http import datetimeToString
 
-from epsilon.extime import Time
+from dateutil.parser import parse as parseTime
 
 from txaws.client.base import BaseClient, BaseQuery, error_wrapper
 from txaws.s3.acls import AccessControlPolicy
 from txaws.s3.model import (
-    Bucket, BucketItem, BucketListing, ItemOwner, RequestPayment)
+    Bucket, BucketItem, BucketListing, ItemOwner, LifecycleConfiguration,
+    LifecycleConfigurationRule, NotificationConfiguration, RequestPayment,
+    VersioningConfiguration, WebsiteConfiguration)
 from txaws.s3.exception import S3Error
 from txaws.service import AWSServiceEndpoint, S3_ENDPOINT
 from txaws.util import XML, calculate_md5
@@ -54,11 +57,18 @@ class URLContext(object):
             if not self.object_name.startswith("/"):
                 path += "/"
             path += self.object_name
+        elif self.bucket is not None and not path.endswith("/"):
+            path += "/"
         return path
 
     def get_url(self):
-        return "%s://%s%s" % (
-            self.endpoint.scheme, self.get_host(), self.get_path())
+        if self.endpoint.port is not None:
+            return "%s://%s:%d%s" % (
+                self.endpoint.scheme, self.get_host(), self.endpoint.port,
+                self.get_path())
+        else:
+            return "%s://%s%s" % (
+                self.endpoint.scheme, self.get_host(), self.get_path())
 
 
 class S3Client(BaseClient):
@@ -90,7 +100,7 @@ class S3Client(BaseClient):
         for bucket_data in root.find("Buckets"):
             name = bucket_data.findtext("Name")
             date_text = bucket_data.findtext("CreationDate")
-            date_time = Time.fromISO8601TimeAndDate(date_text).asDatetime()
+            date_time = parseTime(date_text)
             bucket = Bucket(name, date_time)
             buckets.append(bucket)
         return buckets
@@ -137,8 +147,7 @@ class S3Client(BaseClient):
         for content_data in root.findall("Contents"):
             key = content_data.findtext("Key")
             date_text = content_data.findtext("LastModified")
-            modification_date = Time.fromISO8601TimeAndDate(
-                date_text).asDatetime()
+            modification_date = parseTime(date_text)
             etag = content_data.findtext("ETag")
             size = content_data.findtext("Size")
             storage_class = content_data.findtext("StorageClass")
@@ -174,6 +183,96 @@ class S3Client(BaseClient):
         root = XML(xml_bytes)
         return root.text or ""
 
+    def get_bucket_lifecycle(self, bucket):
+        """
+        Get the lifecycle configuration of a bucket.
+
+        @param bucket: The name of the bucket.
+        @return: A C{Deferred} that will fire with the bucket's lifecycle
+        configuration.
+        """
+        query = self.query_factory(
+            action='GET', creds=self.creds, endpoint=self.endpoint,
+            bucket=bucket, object_name='?lifecycle')
+        return query.submit().addCallback(self._parse_lifecycle_config)
+
+    def _parse_lifecycle_config(self, xml_bytes):
+        """Parse a C{LifecycleConfiguration} XML document."""
+        root = XML(xml_bytes)
+        rules = []
+
+        for content_data in root.findall("Rule"):
+            id = content_data.findtext("ID")
+            prefix = content_data.findtext("Prefix")
+            status = content_data.findtext("Status")
+            expiration = int(content_data.findtext("Expiration/Days"))
+            rules.append(
+                LifecycleConfigurationRule(id, prefix, status, expiration))
+
+        return LifecycleConfiguration(rules)
+
+    def get_bucket_website_config(self, bucket):
+        """
+        Get the website configuration of a bucket.
+
+        @param bucket: The name of the bucket.
+        @return: A C{Deferred} that will fire with the bucket's website
+        configuration.
+        """
+        query = self.query_factory(
+            action='GET', creds=self.creds, endpoint=self.endpoint,
+            bucket=bucket, object_name='?website')
+        return query.submit().addCallback(self._parse_website_config)
+
+    def _parse_website_config(self, xml_bytes):
+        """Parse a C{WebsiteConfiguration} XML document."""
+        root = XML(xml_bytes)
+        index_suffix = root.findtext("IndexDocument/Suffix")
+        error_key = root.findtext("ErrorDocument/Key")
+
+        return WebsiteConfiguration(index_suffix, error_key)
+
+    def get_bucket_notification_config(self, bucket):
+        """
+        Get the notification configuration of a bucket.
+
+        @param bucket: The name of the bucket.
+        @return: A C{Deferred} that will request the bucket's notification
+        configuration.
+        """
+        query = self.query_factory(
+            action='GET', creds=self.creds, endpoint=self.endpoint,
+            bucket=bucket, object_name='?notification')
+        return query.submit().addCallback(self._parse_notification_config)
+
+    def _parse_notification_config(self, xml_bytes):
+        """Parse a C{NotificationConfiguration} XML document."""
+        root = XML(xml_bytes)
+        topic = root.findtext("TopicConfiguration/Topic")
+        event = root.findtext("TopicConfiguration/Event")
+
+        return NotificationConfiguration(topic, event)
+
+    def get_bucket_versioning_config(self, bucket):
+        """
+        Get the versioning configuration of a bucket.
+
+        @param bucket: The name of the bucket.  @return: A C{Deferred} that
+        will request the bucket's versioning configuration.
+        """
+        query = self.query_factory(
+            action='GET', creds=self.creds, endpoint=self.endpoint,
+            bucket=bucket, object_name='?versioning')
+        return query.submit().addCallback(self._parse_versioning_config)
+
+    def _parse_versioning_config(self, xml_bytes):
+        """Parse a C{VersioningConfiguration} XML document."""
+        root = XML(xml_bytes)
+        mfa_delete = root.findtext("MfaDelete")
+        status = root.findtext("Status")
+
+        return VersioningConfiguration(mfa_delete=mfa_delete, status=status)
+
     def get_bucket_acl(self, bucket):
         """
         Get the access control policy for a bucket.
@@ -278,6 +377,16 @@ class S3Client(BaseClient):
             bucket=bucket, object_name=object_name)
         return query.submit()
 
+    def put_object_acl(self, bucket, object_name, access_control_policy):
+        """
+        Set access control policy on an object.
+        """
+        data = access_control_policy.to_xml()
+        query = self.query_factory(
+            action='PUT', creds=self.creds, endpoint=self.endpoint,
+            bucket=bucket, object_name='%s?acl' % object_name, data=data)
+        return query.submit().addCallback(self._parse_acl)
+
     def get_object_acl(self, bucket, object_name):
         """
         Get the access control policy for an object.
@@ -389,12 +498,16 @@ class Query(BaseQuery):
         """
         Get an S3 resource path.
         """
-        resource = "/"
-        if self.bucket:
-            resource += self.bucket
-            if self.object_name:
-                resource += "/%s" % self.object_name
-        return resource
+        path = "/"
+        if self.bucket is not None:
+            path += self.bucket
+        if self.bucket is not None and self.object_name:
+            if not self.object_name.startswith("/"):
+                path += "/"
+            path += self.object_name
+        elif self.bucket is not None and not path.endswith("/"):
+            path += "/"
+        return path
 
     def sign(self, headers):
         """Sign this query using its built in credentials."""
