@@ -1,13 +1,23 @@
-from pytz import UTC
 from cStringIO import StringIO
 from datetime import datetime
 
+from dateutil.tz import tzutc
+
+try:
+    import json
+except ImportError:
+    import simplejson as json
+
 from twisted.trial.unittest import TestCase
+from twisted.python.reflect import safe_str
 
 from txaws.credentials import AWSCredentials
 from txaws.service import AWSServiceEndpoint
 from txaws.ec2.client import Query
+from txaws.server.method import Method
+from txaws.server.registry import Registry
 from txaws.server.resource import QueryAPI
+from txaws.server.exception import APIError
 
 
 class FakeRequest(object):
@@ -55,6 +65,12 @@ class FakeRequest(object):
         return self.written.getvalue()
 
 
+class TestMethod(Method):
+
+    def invoke(self, call):
+        return "data"
+
+
 class TestPrincipal(object):
 
     def __init__(self, creds):
@@ -71,7 +87,6 @@ class TestPrincipal(object):
 
 class TestQueryAPI(QueryAPI):
 
-    actions = ["SomeAction"]
     signature_versions = (1, 2)
     content_type = "text/plain"
 
@@ -79,22 +94,21 @@ class TestQueryAPI(QueryAPI):
         QueryAPI.__init__(self, *args, **kwargs)
         self.principal = None
 
-    def execute(self, call):
-        return "data"
-
     def get_principal(self, access_key):
         if self.principal and self.principal.access_key == access_key:
             return self.principal
 
     def dump_error(self, error, request):
-        return str("%s - %s" % (error.code, error.message))
+        return str("%s - %s" % (error.code, safe_str(error.message)))
 
 
-class QueryAPITest(TestCase):
+class QueryAPITestCase(TestCase):
 
     def setUp(self):
-        super(QueryAPITest, self).setUp()
-        self.api = TestQueryAPI()
+        super(QueryAPITestCase, self).setUp()
+        self.registry = Registry()
+        self.registry.add(TestMethod, action="SomeAction", version=None)
+        self.api = TestQueryAPI(registry=self.registry)
 
     def test_handle(self):
         """
@@ -116,11 +130,46 @@ class QueryAPITest(TestCase):
         self.api.principal = TestPrincipal(creds)
         return self.api.handle(request).addCallback(check)
 
+    def test_handle_with_dump_result(self):
+        """
+        L{QueryAPI.handle} serializes the action result with C{dump_result}.
+        """
+        creds = AWSCredentials("access", "secret")
+        endpoint = AWSServiceEndpoint("http://uri")
+        query = Query(action="SomeAction", creds=creds, endpoint=endpoint)
+        query.sign()
+        request = FakeRequest(query.params, endpoint)
+
+        def check(ignored):
+            self.assertEqual("data", json.loads(request.response))
+
+        self.api.dump_result = json.dumps
+        self.api.principal = TestPrincipal(creds)
+        return self.api.handle(request).addCallback(check)
+
+    def test_handle_with_deprecated_actions(self):
+        """
+        L{QueryAPI.handle} supports the legacy 'actions' attribute.
+        """
+        self.api.actions = ["SomeAction"]
+        creds = AWSCredentials("access", "secret")
+        endpoint = AWSServiceEndpoint("http://uri")
+        query = Query(action="SomeAction", creds=creds, endpoint=endpoint)
+        query.sign()
+        request = FakeRequest(query.params, endpoint)
+
+        def check(ignored):
+            self.assertEqual("data", request.response)
+
+        self.api.principal = TestPrincipal(creds)
+        return self.api.handle(request).addCallback(check)
+
     def test_handle_pass_params_to_call(self):
         """
         L{QueryAPI.handle} creates a L{Call} object with the correct
         parameters.
         """
+        self.registry.add(TestMethod, "SomeAction", "1.2.3")
         creds = AWSCredentials("access", "secret")
         endpoint = AWSServiceEndpoint("http://uri")
         query = Query(action="SomeAction", creds=creds, endpoint=endpoint,
@@ -144,6 +193,29 @@ class QueryAPITest(TestCase):
         self.api.principal = TestPrincipal(creds)
         return self.api.handle(request).addCallback(check)
 
+    def test_handle_ensures_version_is_str(self):
+        """
+        L{QueryAPI.schema} coerces the Version parameter to a str, in order
+        to let URLs built with it be str, as required by urllib.quote in
+        python 2.7.
+        """
+        self.registry.add(TestMethod, "SomeAction", "1.2.3")
+        creds = AWSCredentials("access", "secret")
+        endpoint = AWSServiceEndpoint("http://uri")
+        query = Query(action="SomeAction", creds=creds, endpoint=endpoint,
+                      other_params={"Version": u"1.2.3"})
+        query.sign()
+        request = FakeRequest(query.params, endpoint)
+
+        def execute(call):
+            self.assertEqual("1.2.3", call.version)
+            self.assertIsInstance(call.version, str)
+            return "ok"
+
+        self.api.execute = execute
+        self.api.principal = TestPrincipal(creds)
+        return self.api.handle(request)
+
     def test_handle_empty_request(self):
         """
         If an empty request is received a message describing the API is
@@ -198,7 +270,8 @@ class QueryAPITest(TestCase):
         request = FakeRequest(query.params, endpoint)
 
         def check(ignored):
-            self.flushLoggedErrors()
+            errors = self.flushLoggedErrors()
+            self.assertEquals(0, len(errors))
             self.assertEqual("InvalidSignature - SignatureVersion '2' "
                              "not supported", request.response)
             self.assertEqual(403, request.code)
@@ -220,7 +293,8 @@ class QueryAPITest(TestCase):
         self.api.execute = lambda call: 1 / 0
 
         def check(ignored):
-            self.flushLoggedErrors()
+            errors = self.flushLoggedErrors()
+            self.assertEquals(1, len(errors))
             self.assertTrue(request.finished)
             self.assertEqual("integer division or modulo by zero",
                              request.response)
@@ -229,6 +303,31 @@ class QueryAPITest(TestCase):
         self.api.principal = TestPrincipal(creds)
         return self.api.handle(request).addCallback(check)
 
+    def test_handle_500_api_error(self):
+        """
+        If an L{APIError} is raised with a status code superior or equal to
+        500, the error is logged on the server side.
+        """
+        creds = AWSCredentials("access", "secret")
+        endpoint = AWSServiceEndpoint("http://uri")
+        query = Query(action="SomeAction", creds=creds, endpoint=endpoint)
+        query.sign()
+        request = FakeRequest(query.params, endpoint)
+
+        def fail_execute(call):
+            raise APIError(500, response="oops")
+        self.api.execute = fail_execute
+
+        def check(ignored):
+            errors = self.flushLoggedErrors()
+            self.assertEquals(1, len(errors))
+            self.assertTrue(request.finished)
+            self.assertEqual("oops", request.response)
+            self.assertEqual(500, request.code)
+
+        self.api.principal = TestPrincipal(creds)
+        return self.api.handle(request).addCallback(check)
+
     def test_handle_with_parameter_error(self):
         """
         If an error occurs while parsing the parameters, L{QueryAPI.handle}
@@ -242,15 +341,67 @@ class QueryAPITest(TestCase):
         request = FakeRequest(query.params, endpoint)
 
         def check(ignored):
-            self.flushLoggedErrors()
+            errors = self.flushLoggedErrors()
+            self.assertEquals(0, len(errors))
             self.assertEqual("MissingParameter - The request must contain "
                              "the parameter Action", request.response)
             self.assertEqual(400, request.code)
 
         return self.api.handle(request).addCallback(check)
 
+    def test_handle_unicode_api_error(self):
+        """
+        If an L{APIError} contains a unicode message, L{QueryAPI} is able to
+        protect itself from it.
+        """
+        creds = AWSCredentials("access", "secret")
+        endpoint = AWSServiceEndpoint("http://uri")
+        query = Query(action="SomeAction", creds=creds, endpoint=endpoint)
+        query.sign()
+        request = FakeRequest(query.params, endpoint)
+
+        def fail_execute(call):
+            raise APIError(400, code="LangError",
+                           message=u"\N{HIRAGANA LETTER A}dvanced")
+        self.api.execute = fail_execute
+
+        def check(ignored):
+            errors = self.flushLoggedErrors()
+            self.assertEquals(0, len(errors))
+            self.assertTrue(request.finished)
+            self.assertTrue(request.response.startswith("LangError"))
+            self.assertEqual(400, request.code)
+
+        self.api.principal = TestPrincipal(creds)
+        return self.api.handle(request).addCallback(check)
+
+    def test_handle_unicode_error(self):
+        """
+        If an arbitrary error raised by an API method contains a unicode
+        message, L{QueryAPI} is able to protect itself from it.
+        """
+        creds = AWSCredentials("access", "secret")
+        endpoint = AWSServiceEndpoint("http://uri")
+        query = Query(action="SomeAction", creds=creds, endpoint=endpoint)
+        query.sign()
+        request = FakeRequest(query.params, endpoint)
+
+        def fail_execute(call):
+            raise ValueError(u"\N{HIRAGANA LETTER A}dvanced")
+        self.api.execute = fail_execute
+
+        def check(ignored):
+            errors = self.flushLoggedErrors()
+            self.assertEquals(1, len(errors))
+            self.assertTrue(request.finished)
+            self.assertIn("ValueError", request.response)
+            self.assertEqual(500, request.code)
+
+        self.api.principal = TestPrincipal(creds)
+        return self.api.handle(request).addCallback(check)
+
     def test_handle_with_unsupported_action(self):
-        """Only actions listed in L{QueryAPI.actions} are supported."""
+        """Only actions registered in the L{Registry} are supported."""
         creds = AWSCredentials("access", "secret")
         endpoint = AWSServiceEndpoint("http://uri")
         query = Query(action="FooBar", creds=creds, endpoint=endpoint)
@@ -258,7 +409,54 @@ class QueryAPITest(TestCase):
         request = FakeRequest(query.params, endpoint)
 
         def check(ignored):
-            self.flushLoggedErrors()
+            errors = self.flushLoggedErrors()
+            self.assertEquals(0, len(errors))
+            self.assertEqual("InvalidAction - The action FooBar is not valid"
+                             " for this web service.", request.response)
+            self.assertEqual(400, request.code)
+
+        return self.api.handle(request).addCallback(check)
+
+    def test_handle_non_available_method(self):
+        """Only actions registered in the L{Registry} are supported."""
+
+        class NonAvailableMethod(Method):
+
+            def is_available(self):
+                return False
+
+        self.registry.add(NonAvailableMethod, action="CantDoIt")
+        creds = AWSCredentials("access", "secret")
+        endpoint = AWSServiceEndpoint("http://uri")
+        query = Query(action="CantDoIt", creds=creds, endpoint=endpoint)
+        query.sign()
+        request = FakeRequest(query.params, endpoint)
+
+        def check(ignored):
+            errors = self.flushLoggedErrors()
+            self.assertEquals(0, len(errors))
+            self.assertEqual("InvalidAction - The action CantDoIt is not "
+                             "valid for this web service.", request.response)
+            self.assertEqual(400, request.code)
+
+        self.api.principal = TestPrincipal(creds)
+        return self.api.handle(request).addCallback(check)
+
+    def test_handle_with_deprecated_actions_and_unsupported_action(self):
+        """
+        If the deprecated L{QueryAPI.actions} attribute is set, it will be
+        used for looking up supported actions.
+        """
+        self.api.actions = ["SomeAction"]
+        creds = AWSCredentials("access", "secret")
+        endpoint = AWSServiceEndpoint("http://uri")
+        query = Query(action="FooBar", creds=creds, endpoint=endpoint)
+        query.sign()
+        request = FakeRequest(query.params, endpoint)
+
+        def check(ignored):
+            errors = self.flushLoggedErrors()
+            self.assertEquals(0, len(errors))
             self.assertEqual("InvalidAction - The action FooBar is not valid"
                              " for this web service.", request.response)
             self.assertEqual(400, request.code)
@@ -277,7 +475,8 @@ class QueryAPITest(TestCase):
         request = FakeRequest(query.params, endpoint)
 
         def check(ignored):
-            self.flushLoggedErrors()
+            errors = self.flushLoggedErrors()
+            self.assertEquals(0, len(errors))
             self.assertEqual("AuthFailure - No user with access key 'access'",
                              request.response)
             self.assertEqual(401, request.code)
@@ -297,7 +496,8 @@ class QueryAPITest(TestCase):
         request = FakeRequest(query.params, endpoint)
 
         def check(ignored):
-            self.flushLoggedErrors()
+            errors = self.flushLoggedErrors()
+            self.assertEquals(0, len(errors))
             self.assertEqual("SignatureDoesNotMatch - The request signature "
                              "we calculated does not match the signature you "
                              "provided. Check your key and signing method.",
@@ -321,7 +521,8 @@ class QueryAPITest(TestCase):
         request = FakeRequest(query.params, endpoint)
 
         def check(ignored):
-            self.flushLoggedErrors()
+            errors = self.flushLoggedErrors()
+            self.assertEquals(0, len(errors))
             self.assertEqual(
                 "InvalidParameterCombination - The parameter Timestamp"
                 " cannot be used with the parameter Expires",
@@ -346,7 +547,7 @@ class QueryAPITest(TestCase):
             self.assertEqual("data", request.response)
             self.assertEqual(200, request.code)
 
-        now = datetime(2009, 12, 31, tzinfo=UTC)
+        now = datetime(2009, 12, 31, tzinfo=tzutc())
         self.api.get_utc_time = lambda: now
         self.api.principal = TestPrincipal(creds)
         return self.api.handle(request).addCallback(check)
@@ -364,13 +565,14 @@ class QueryAPITest(TestCase):
         request = FakeRequest(query.params, endpoint)
 
         def check(ignored):
-            self.flushLoggedErrors()
+            errors = self.flushLoggedErrors()
+            self.assertEquals(0, len(errors))
             self.assertEqual(
                 "RequestExpired - Request has expired. Expires date is"
                 " 2010-01-01T12:00:00Z", request.response)
             self.assertEqual(400, request.code)
 
-        now = datetime(2010, 1, 1, 12, 0, 1, tzinfo=UTC)
+        now = datetime(2010, 1, 1, 12, 0, 1, tzinfo=tzutc())
         self.api.get_utc_time = lambda: now
         return self.api.handle(request).addCallback(check)
 
