@@ -1,6 +1,8 @@
+# -*- coding: utf-8 -*-
+
 from datetime import datetime
 
-from pytz import UTC, FixedOffset
+from dateutil.tz import tzutc, tzoffset
 
 from twisted.trial.unittest import TestCase
 
@@ -9,7 +11,7 @@ from txaws.server.schema import (
     Arguments, Bool, Date, Enum, Integer, Parameter, RawStr, Schema, Unicode)
 
 
-class ArgumentsTest(TestCase):
+class ArgumentsTestCase(TestCase):
 
     def test_instantiate_empty(self):
         """Creating an L{Arguments} object."""
@@ -54,7 +56,7 @@ class ArgumentsTest(TestCase):
         self.assertEqual("egg", arguments.foo[0])
 
 
-class ParameterTest(TestCase):
+class ParameterTestCase(TestCase):
 
     def test_coerce(self):
         """
@@ -102,6 +104,19 @@ class ParameterTest(TestCase):
         self.assertEqual("InvalidParameterValue", error.code)
         self.assertEqual("Invalid integer value foo", error.message)
 
+    def test_coerce_with_parameter_error_unicode(self):
+        """
+        L{Parameter.coerce} raises an L{APIError} if an invalid value is
+        passed as request argument and parameter value is unicode.
+        """
+        parameter = Parameter("Test")
+        parameter.parse = lambda value: int(value)
+        parameter.kind = "integer"
+        error = self.assertRaises(APIError, parameter.coerce, "citt\xc3\xa1")
+        self.assertEqual(400, error.status)
+        self.assertEqual("InvalidParameterValue", error.code)
+        self.assertEqual(u"Invalid integer value cittá", error.message)
+
     def test_coerce_with_empty_strings(self):
         """
         L{Parameter.coerce} returns C{None} if the value is an empty string and
@@ -150,14 +165,41 @@ class ParameterTest(TestCase):
         self.assertEqual("Value (longish) for parameter Test is invalid.  "
                          "3 should be enough for anybody", error.message)
 
+    def test_validator_invalid(self):
+        """
+        L{Parameter.coerce} raises an error if the validator returns False.
+        """
+        parameter = Parameter("Test", validator=lambda _: False)
+        parameter.parse = lambda value: value
+        parameter.kind = "test_parameter"
+        error = self.assertRaises(APIError, parameter.coerce, "foo")
+        self.assertEqual(400, error.status)
+        self.assertEqual("InvalidParameterValue", error.code)
+        self.assertEqual("Invalid test_parameter value foo", error.message)
 
-class UnicodeTest(TestCase):
+    def test_validator_valid(self):
+        """
+        L{Parameter.coerce} returns the correct value if validator returns
+        True.
+        """
+        parameter = Parameter("Test", validator=lambda _: True)
+        parameter.parse = lambda value: value
+        parameter.kind = "test_parameter"
+        self.assertEqual("foo", parameter.coerce("foo"))
+
+
+class UnicodeTestCase(TestCase):
 
     def test_parse(self):
         """L{Unicode.parse} converts the given raw C{value} to C{unicode}."""
         parameter = Unicode("Test")
         self.assertEqual(u"foo", parameter.parse("foo"))
 
+    def test_parse_unicode(self):
+        """L{Unicode.parse} works with unicode input."""
+        parameter = Unicode("Test")
+        self.assertEqual(u"cittá", parameter.parse("citt\xc3\xa1"))
+
     def test_format(self):
         """L{Unicode.format} encodes the given C{unicode} with utf-8."""
         parameter = Unicode("Test")
@@ -179,8 +221,18 @@ class UnicodeTest(TestCase):
         self.assertEqual(400, error.status)
         self.assertEqual("InvalidParameterValue", error.code)
 
+    def test_invalid_unicode(self):
+        """
+        The L{Unicode} parameter returns an error with invalid unicode data.
+        """
+        parameter = Unicode("Test")
+        error = self.assertRaises(APIError, parameter.coerce, "Test\x95Error")
+        self.assertIn(u"Invalid unicode value", error.message)
+        self.assertEqual(400, error.status)
+        self.assertEqual("InvalidParameterValue", error.code)
 
-class RawStrTest(TestCase):
+
+class RawStrTestCase(TestCase):
 
     def test_parse(self):
         """L{RawStr.parse} checks that the given raw C{value} is a string."""
@@ -195,7 +247,7 @@ class RawStrTest(TestCase):
         self.assertTrue(isinstance(value, str))
 
 
-class IntegerTest(TestCase):
+class IntegerTestCase(TestCase):
 
     def test_parse(self):
         """L{Integer.parse} converts the given raw C{value} to C{int}."""
@@ -205,15 +257,44 @@ class IntegerTest(TestCase):
     def test_parse_with_negative(self):
         """L{Integer.parse} converts the given raw C{value} to C{int}."""
         parameter = Integer("Test")
-        self.assertRaises(ValueError, parameter.parse, "-1")
+        error = self.assertRaises(APIError, parameter.coerce, "-1")
+        self.assertEqual(400, error.status)
+        self.assertEqual("InvalidParameterValue", error.code)
+        self.assertIn("Value must be at least 0.", error.message)
 
     def test_format(self):
         """L{Integer.format} converts the given integer to a string."""
         parameter = Integer("Test")
         self.assertEqual("123", parameter.format(123))
 
+    def test_min_and_max(self):
+        """The L{Integer} parameter properly supports ranges."""
+        parameter = Integer("Test", min=2, max=4)
 
-class BoolTest(TestCase):
+        error = self.assertRaises(APIError, parameter.coerce, "1")
+        self.assertEqual(400, error.status)
+        self.assertEqual("InvalidParameterValue", error.code)
+        self.assertIn("Value must be at least 2.", error.message)
+
+        error = self.assertRaises(APIError, parameter.coerce, "5")
+        self.assertIn("Value exceeds maximum of 4.", error.message)
+        self.assertEqual(400, error.status)
+        self.assertEqual("InvalidParameterValue", error.code)
+
+    def test_non_integer_string(self):
+        """
+        The L{Integer} parameter raises an L{APIError} when passed non-int
+        values (in this case, a string).
+        """
+        garbage = "blah"
+        parameter = Integer("Test")
+        error = self.assertRaises(APIError, parameter.coerce, garbage)
+        self.assertEqual(400, error.status)
+        self.assertEqual("InvalidParameterValue", error.code)
+        self.assertIn("Invalid integer value %s" % garbage, error.message)
+
+
+class BoolTestCase(TestCase):
 
     def test_parse(self):
         """L{Bool.parse} converts 'true' to C{True}."""
@@ -240,7 +321,7 @@ class BoolTest(TestCase):
         self.assertEqual("false", parameter.format(False))
 
 
-class EnumTest(TestCase):
+class EnumTestCase(TestCase):
 
     def test_parse(self):
         """L{Enum.parse} accepts a map for translating values."""
@@ -261,12 +342,12 @@ class EnumTest(TestCase):
         self.assertEqual("foo", parameter.format("bar"))
 
 
-class DateTest(TestCase):
+class DateTestCase(TestCase):
 
     def test_parse(self):
         """L{Date.parse checks that the given raw C{value} is a date/time."""
         parameter = Date("Test")
-        date = datetime(2010, 9, 15, 23, 59, 59, tzinfo=UTC)
+        date = datetime(2010, 9, 15, 23, 59, 59, tzinfo=tzutc())
         self.assertEqual(date, parameter.parse("2010-09-15T23:59:59Z"))
 
     def test_format(self):
@@ -276,11 +357,11 @@ class DateTest(TestCase):
         """
         parameter = Date("Test")
         date = datetime(2010, 9, 15, 23, 59, 59,
-                        tzinfo=FixedOffset(120))
+                        tzinfo=tzoffset('UTC', 120 * 60))
         self.assertEqual("2010-09-15T21:59:59Z", parameter.format(date))
 
 
-class SchemaTest(TestCase):
+class SchemaTestCase(TestCase):
 
     def test_extract(self):
         """
@@ -491,3 +572,21 @@ class SchemaTest(TestCase):
         """
         schema = Schema(Integer("count"))
         self.assertRaises(RuntimeError, schema.bundle, name="foo")
+
+    def test_add_single_extra_schema_item(self):
+        """New Parameters can be added to the Schema."""
+        schema = Schema(Unicode("name"))
+        schema = schema.extend(Unicode("computer"))
+        arguments, _ = schema.extract({"name": "value", "computer": "testing"})
+        self.assertEqual(u"value", arguments.name)
+        self.assertEqual("testing", arguments.computer)
+
+    def test_add_extra_schema_items(self):
+        """A list of new Parameters can be added to the Schema."""
+        schema = Schema(Unicode("name"))
+        schema = schema.extend(Unicode("computer"), Integer("count"))
+        arguments, _ = schema.extract({"name": "value", "computer": "testing",
+                                       "count": "5"})
+        self.assertEqual(u"value", arguments.name)
+        self.assertEqual("testing", arguments.computer)
+        self.assertEqual(5, arguments.count)
