@@ -3,29 +3,35 @@
 # Licenced under the txaws licence available at /LICENSE in the txaws source.
 
 from txaws.credentials import AWSCredentials
+from txaws import regions
 from txaws.util import parse
 
+
 __all__ = ["AWSServiceEndpoint", "AWSServiceRegion", "REGION_US", "REGION_EU"]
 
 
-REGION_US = "US"
-REGION_EU = "EU"
-EC2_ENDPOINT_US = "https://us-east-1.ec2.amazonaws.com/"
-EC2_ENDPOINT_EU = "https://eu-west-1.ec2.amazonaws.com/"
-S3_ENDPOINT = "https://s3.amazonaws.com/"
+# These old variable names are maintained for backwards compatibility.
+REGION_US = regions.REGION_US
+REGION_EU = regions.REGION_EU
+EC2_ENDPOINT_US = regions.EC2_ENDPOINT_US
+EC2_ENDPOINT_EU = regions.EC2_ENDPOINT_EU
+S3_ENDPOINT = regions.S3_ENDPOINT
 
 
 class AWSServiceEndpoint(object):
     """
     @param uri: The URL for the service.
     @param method: The HTTP method used when accessing a service.
+    @param ssl_hostname_verification: Whether or not SSL hotname verification
+        will be done when connecting to the endpoint.
     """
 
-    def __init__(self, uri="", method="GET"):
+    def __init__(self, uri="", method="GET", ssl_hostname_verification=False):
         self.host = ""
         self.port = None
         self.path = "/"
         self.method = method
+        self.ssl_hostname_verification = ssl_hostname_verification
         self._parse_uri(uri)
         if not self.scheme:
             self.scheme = "http"
