@@ -22,8 +22,11 @@ sample_required_describe_instances_result = """\
             <code>16</code>
             <name>running</name>
           </instanceState>
-          <privateDnsName>domU-12-31-39-03-15-11.compute-1.internal</privateDnsName>
+          <privateDnsName>domU-12-31-39-03-15-11.compute-1.internal\
+</privateDnsName>
           <dnsName>ec2-75-101-245-65.compute-1.amazonaws.com</dnsName>
+          <privateIpAddress>10.0.0.1</privateIpAddress>
+          <ipAddress>75.101.245.65</ipAddress>
           <instanceType>c1.xlarge</instanceType>
           <launchTime>2009-04-27T02:23:18.000Z</launchTime>
           <placement>
@@ -58,8 +61,11 @@ sample_describe_instances_result = """\
             <code>16</code>
             <name>running</name>
           </instanceState>
-          <privateDnsName>domU-12-31-39-03-15-11.compute-1.internal</privateDnsName>
+          <privateDnsName>domU-12-31-39-03-15-11.compute-1.internal\
+</privateDnsName>
           <dnsName>ec2-75-101-245-65.compute-1.amazonaws.com</dnsName>
+          <privateIpAddress>10.0.0.1</privateIpAddress>
+          <ipAddress>75.101.245.65</ipAddress>
           <reason/>
           <keyName>keyname</keyName>
           <amiLaunchIndex>0</amiLaunchIndex>
@@ -153,10 +159,10 @@ sample_terminate_instances_result = """\
   <instancesSet>
     <item>
       <instanceId>i-1234</instanceId>
-      <shutdownState>
+      <currentState>
         <code>32</code>
         <name>shutting-down</name>
-      </shutdownState>
+      </currentState>
       <previousState>
         <code>16</code>
         <name>running</name>
@@ -164,10 +170,10 @@ sample_terminate_instances_result = """\
     </item>
     <item>
       <instanceId>i-5678</instanceId>
-      <shutdownState>
+      <currentState>
         <code>32</code>
         <name>shutting-down</name>
-      </shutdownState>
+      </currentState>
       <previousState>
         <code>32</code>
         <name>shutting-down</name>
@@ -178,6 +184,43 @@ sample_terminate_instances_result = """\
 """ % (version.ec2_api,)
 
 
+sample_describe_security_groups_with_openstack = """\
+<?xml version="1.0"?>
+<DescribeSecurityGroupsResponse xmlns="http://ec2.amazonaws.com/doc/%s/">
+  <requestId>7d4e4dbd-0a33-4d3a-864a-b5ce0f1c9cbf</requestId>
+  <securityGroupInfo>
+    <item>
+      <ipPermissions>
+        <item>
+          <toPort>22</toPort>
+          <ipProtocol>tcp</ipProtocol>
+          <ipRanges>
+             <item><cidrIp>0.0.0.0/0</cidrIp></item>
+          </ipRanges>
+          <groups/>
+          <fromPort>22</fromPort>
+        </item>
+        <item>
+         <toPort/>
+         <ipProtocol/>
+         <ipRanges/>
+         <groups>
+            <item>
+              <groupName>WebServers</groupName>
+              <userId>UYY3TLBUXIEON5NQVUUX6OMPWBZIQNFM</userId>
+            </item>
+         </groups>
+         <fromPort/>
+        </item>
+      </ipPermissions>
+      <groupName>WebServers</groupName>
+      <groupDescription>Web servers</groupDescription>
+      <ownerId>UYY3TLBUXIEON5NQVUUX6OMPWBZIQNFM</ownerId>
+    </item>
+  </securityGroupInfo>
+</DescribeSecurityGroupsResponse>
+""" % (version.ec2_api,)
+
 sample_describe_security_groups_result = """\
 <?xml version="1.0"?>
 <DescribeSecurityGroupsResponse xmlns="http://ec2.amazonaws.com/doc/%s/">
@@ -457,7 +500,8 @@ sample_delete_security_group_failure = """\
   <Errors>
     <Error>
       <Code>InvalidGroup.InUse</Code>
-      <Message>Group groupID1:GroupReferredTo is used by groups: groupID2:UsingGroup</Message>
+      <Message>Group groupID1:GroupReferredTo is used by groups: \
+groupID2:UsingGroup</Message>
     </Error>
   </Errors>
   <RequestID>9a6df05f-9c27-47aa-81d8-6619689210cc</RequestID>
@@ -466,7 +510,8 @@ sample_delete_security_group_failure = """\
 
 
 sample_authorize_security_group = """\
-<AuthorizeSecurityGroupIngressResponse xmlns="http://ec2.amazonaws.com/doc/%s/">
+<AuthorizeSecurityGroupIngressResponse\
+ xmlns="http://ec2.amazonaws.com/doc/%s/">
   <return>true</return>
 </AuthorizeSecurityGroupIngressResponse>
 """ % (version.ec2_api,)
@@ -613,7 +658,8 @@ sample_single_describe_keypairs_result = """\
   <keySet>
     <item>
       <keyName>gsg-keypair</keyName>
-      <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:ca:9f:f5:f1:6f</keyFingerprint>
+      <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:\
+ca:9f:f5:f1:6f</keyFingerprint>
     </item>
   </keySet>
 </DescribeKeyPairsResponse>
@@ -626,11 +672,13 @@ sample_multiple_describe_keypairs_result = """\
   <keySet>
     <item>
       <keyName>gsg-keypair-1</keyName>
-      <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:ca:9f:f5:f1:6f</keyFingerprint>
+      <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:\
+ca:9f:f5:f1:6f</keyFingerprint>
     </item>
     <item>
       <keyName>gsg-keypair-2</keyName>
-      <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:ca:9f:f5:f1:70</keyFingerprint>
+      <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:\
+ca:9f:f5:f1:70</keyFingerprint>
     </item>
   </keySet>
 </DescribeKeyPairsResponse>
@@ -641,7 +689,8 @@ sample_create_keypair_result = """\
 <?xml version="1.0"?>
 <CreateKeyPairResponse xmlns="http://ec2.amazonaws.com/doc/%s/">
   <keyName>example-key-name</keyName>
-  <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:ca:9f:f5:f1:6f</keyFingerprint>
+  <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:\
+ca:9f:f5:f1:6f</keyFingerprint>
   <keyMaterial>-----BEGIN RSA PRIVATE KEY-----
 MIIEoQIBAAKCAQBuLFg5ujHrtm1jnutSuoO8Xe56LlT+HM8v/xkaa39EstM3/aFxTHgElQiJLChp
 HungXQ29VTc8rc1bW0lkdi23OH5eqkMHGhvEwqa0HWASUMll4o3o/IX+0f2UcPoKCOVUR+jx71Sg
@@ -710,7 +759,8 @@ sample_import_keypair_result = """\
 <?xml version="1.0"?>
 <ImportKeyPairResponse xmlns="http://ec2.amazonaws.com/doc/%s/">
   <keyName>example-key-name</keyName>
-  <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:ca:9f:f5:f1:6f</keyFingerprint>
+  <keyFingerprint>1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:\
+ca:9f:f5:f1:6f</keyFingerprint>
 </ImportKeyPairResponse>
 """ % (version.ec2_api,)
 
@@ -800,7 +850,8 @@ sample_invalid_client_token_result = """\
   <Errors>
     <Error>
       <Code>InvalidClientTokenId</Code>
-      <Message>The AWS Access Key Id you provided does not exist in our records.</Message>
+      <Message>The AWS Access Key Id you provided does not exist in our\
+ records.</Message>
     </Error>
   </Errors>
   <RequestID>47bfd77d-78d6-446d-be0d-f7621795dded</RequestID>
@@ -828,7 +879,8 @@ sample_server_internal_error_result = """\
   <Code>InternalError</Code>
   <Message>We encountered an internal error. Please try again.</Message>
   <RequestID>A2A7E5395E27DFBB</RequestID>
-  <HostID>f691zulHNsUqonsZkjhILnvWwD3ZnmOM4ObM1wXTc6xuS3GzPmjArp8QC/sGsn6K</HostID>
+  <HostID>f691zulHNsUqonsZkjhILnvWwD3ZnmOM4ObM1wXTc6xuS3GzPmjArp8QC/sGsn6K\
+</HostID>
 </Error>
 """
 
@@ -889,7 +941,8 @@ sample_get_bucket_result = """\
 
 
 sample_get_bucket_location_result = """\
-<LocationConstraint xmlns="http://s3.amazonaws.com/doc/2006-03-01/">EU</LocationConstraint>
+<LocationConstraint xmlns="http://s3.amazonaws.com/doc/2006-03-01/">EU\
+</LocationConstraint>
 
 """
 sample_request_payment = """\
@@ -903,12 +956,18 @@ sample_s3_signature_mismatch = """\
 <?xml version="1.0" encoding="UTF-8"?>
 <Error>
   <Code>SignatureDoesNotMatch</Code>
-  <Message>The request signature we calculated does not match the signature you provided. Check your key and signing method.</Message>
-  <StringToSignBytes>47 45 54 0a 31 42 32 4d 32 59 38 41 73 67 54 70 67 41 6d 59 37 50 68 43 66 67 3d 3d 0a 0a 54 68 75 2c 20 30 35 20 4e 6f 76 20 32 30 30 39 20 32 31 3a 33 33 3a 32 39 20 47 4d 54 0a 2f</StringToSignBytes>
+  <Message>The request signature we calculated does not match the signature\
+ you provided. Check your key and signing method.</Message>
+  <StringToSignBytes>47 45 54 0a 31 42 32 4d 32 59 38 41 73 67 54 70 67 41 6d\
+ 59 37 50 68 43 66 67 3d 3d 0a 0a 54 68 75 2c 20 30 35 20 4e 6f 76 20 32 30\
+ 30 39 20 32 31 3a 33 33 3a 32 39 20 47 4d 54 0a 2f</StringToSignBytes>
   <RequestId>AB9216C8640751B2</RequestId>
-  <HostId>sAPBpmFdsOsgUUwtSLsiT6KIwP1mPbmrYY0xUoahzJE263qmABkTaqzGhHddgOq5</HostId>
-  <SignatureProvided>ltowhdrbjaQ8dQc9VS5MxzJfsPJZi0BZHEzJC3r9pzU=</SignatureProvided>
-  <StringToSign>GET\n1B2M2Y8AsgTpgAmY7PhCfg==\n\nThu, 05 Nov 2009 21:33:29 GMT\n/</StringToSign>
+  <HostId>sAPBpmFdsOsgUUwtSLsiT6KIwP1mPbmrYY0xUoahzJE263qmABkTaqzGhHddgOq5\
+</HostId>
+  <SignatureProvided>ltowhdrbjaQ8dQc9VS5MxzJfsPJZi0BZHEzJC3r9pzU=
+</SignatureProvided>
+  <StringToSign>GET\n1B2M2Y8AsgTpgAmY7PhCfg==\n\nThu, 05 Nov 2009 21:33:29\
+ GMT\n/</StringToSign>
   <AWSAccessKeyId>SOMEKEYID</AWSAccessKeyId>
 </Error>
 """
@@ -918,9 +977,11 @@ sample_s3_invalid_access_key_result = """\
 <?xml version="1.0" encoding="UTF-8"?>
 <Error>
   <Code>InvalidAccessKeyId</Code>
-  <Message>The AWS Access Key Id you provided does not exist in our records.</Message>
+  <Message>The AWS Access Key Id you provided does not exist in our records.\
+</Message>
   <RequestId>0223AD81A94821CE</RequestId>
-  <HostId>HAw5g9P1VkN8ztgLKFTK20CY5LmCfTwXcSths1O7UQV6NuJx2P4tmFnpuOsziwOE</HostId>
+  <HostId>HAw5g9P1VkN8ztgLKFTK20CY5LmCfTwXcSths1O7UQV6NuJx2P4tmFnpuOsziwOE\
+</HostId>
   <AWSAccessKeyId>SOMEKEYID</AWSAccessKeyId>
 </Error>
 """
@@ -933,14 +994,16 @@ sample_access_control_policy_result = """\
   </Owner>
   <AccessControlList>
     <Grant>
-      <Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="CanonicalUser">
+      <Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"\
+ xsi:type="CanonicalUser">
         <ID>8a6925ce4adf588a4f21c32aa379004fef</ID>
         <DisplayName>foo@example.net</DisplayName>
       </Grantee>
       <Permission>FULL_CONTROL</Permission>
     </Grant>
     <Grant>
-      <Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="CanonicalUser">
+      <Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"\
+ xsi:type="CanonicalUser">
         <ID>8a6925ce4adf588a4f21c32aa37900feed</ID>
         <DisplayName>bar@example.net</DisplayName>
       </Grantee>
@@ -949,3 +1012,80 @@ sample_access_control_policy_result = """\
   </AccessControlList>
 </AccessControlPolicy>"""
 
+sample_s3_get_bucket_lifecycle_result = """\
+<?xml version="1.0" encoding="UTF-8"?>
+<LifecycleConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
+   <Rule>
+      <ID>30-day-log-deletion-rule</ID>
+      <Prefix>logs</Prefix>
+      <Status>Enabled</Status>
+      <Expiration>
+         <Days>30</Days>
+      </Expiration>
+   </Rule>
+</LifecycleConfiguration>"""
+
+sample_s3_get_bucket_lifecycle_multiple_rules_result = """\
+<?xml version="1.0" encoding="UTF-8"?>
+<LifecycleConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
+   <Rule>
+      <ID>30-day-log-deletion-rule</ID>
+      <Prefix>logs</Prefix>
+      <Status>Enabled</Status>
+      <Expiration>
+         <Days>30</Days>
+      </Expiration>
+   </Rule>
+   <Rule>
+      <ID>another-id</ID>
+      <Prefix>another-logs</Prefix>
+      <Status>Disabled</Status>
+      <Expiration>
+         <Days>37</Days>
+      </Expiration>
+   </Rule>
+</LifecycleConfiguration>"""
+
+sample_s3_get_bucket_website_result = """\
+<?xml version="1.0" encoding="UTF-8"?>
+<WebsiteConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
+  <IndexDocument>
+    <Suffix>index.html</Suffix>
+  </IndexDocument>
+  <ErrorDocument>
+    <Key>404.html</Key>
+  </ErrorDocument>
+</WebsiteConfiguration>"""
+
+sample_s3_get_bucket_website_no_error_result = """\
+<?xml version="1.0" encoding="UTF-8"?>
+<WebsiteConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
+  <IndexDocument>
+    <Suffix>index.html</Suffix>
+  </IndexDocument>
+</WebsiteConfiguration>"""
+
+sample_s3_get_bucket_notification_result = """\
+<NotificationConfiguration />"""
+
+sample_s3_get_bucket_notification_with_topic_result = """\
+<NotificationConfiguration>
+   <TopicConfiguration>
+       <Topic>arn:aws:sns:us-east-1:123456789012:myTopic</Topic>
+       <Event>s3:ReducedRedundancyLostObject</Event>
+   </TopicConfiguration>
+</NotificationConfiguration>"""
+
+sample_s3_get_bucket_versioning_result = """\
+<VersioningConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/"/>"""
+
+sample_s3_get_bucket_versioning_enabled_result = """\
+<VersioningConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
+  <Status>Enabled</Status>
+</VersioningConfiguration>"""
+
+sample_s3_get_bucket_versioning_mfa_disabled_result = """\
+<VersioningConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
+  <Status>Enabled</Status>
+  <MfaDelete>Disabled</MfaDelete>
+</VersioningConfiguration>"""
