@@ -6,7 +6,7 @@ from txaws.ec2.client import EC2Client
 try:
     from txaws.s3.client import S3Client
 except ImportError:
-    s3clientSkip = ("S3Client couldn't be imported (perhaps because epsilon, "
+    s3clientSkip = ("S3Client couldn't be imported (perhaps because dateutil, "
                     "on which it depends, isn't present)")
 else:
     s3clientSkip = None
